# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

import sphinx_rtd_theme

# -- Path setup --------------------------------------------------------------


sys.path.insert(0, os.path.abspath(".."))

# -- Project information -----------------------------------------------------

project = "fastcountvectorizer"
copyright = "2020, Santiago M. Mola"
author = "Santiago M. Mola"

# The full version, including alpha/beta/rc tags
release = "0.1.0"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = []

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# Until Sphinx 2.0, it defaulted to contents.
master_doc = "index"

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_rtd_theme"
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

html_theme_options = {
    "navigation_depth": 4,
    "collapse_navigation": False,
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ["_static"]

html_sidebars = {
    "**": ["globaltoc.html", "relations.html", "sourcelink.html", "searchbox.html"]
}

# Enable sphinx.ext.napoleon for numpy-style docstrins
extensions += ["sphinx.ext.napoleon"]

# autodoc
extensions += ["sphinx.ext.autodoc"]
autodoc_mock_imports = ["fastcountvectorizer._ext"]


def autodoc_skip_member(app, what, name, obj, skip, options):
    if name == "fastcountvectorizer.fastcountvectorizer":
        return True
    return None


def setup(app):
    app.connect("autodoc-skip-member", autodoc_skip_member)
